/*************************************************************************
 * Copyright(c) 2013~2020  El Condor
 *  
 * Version 1.2.0 17 August 2020
 ************************************************************************/
#include <stdio.h>
#include <string.h>
#include "handleEvents.h"
#define LENGTHWORKBUFFER 200
char workBuffer [LENGTHWORKBUFFER+1];
int led = 13;
void listCommands();
// every,next,duration,status,command,fnzActions,name
void handle_command(cronStructure &event);
void handle_atStart(cronStructure &event);
cronStructure cron[] = {
        {0,0,0,EVENT_ENABLED,(fnzPointer) handle_atStart,"atStart"},        // start on command
        {4,0,0,EVENT_ENABLED,(fnzPointer) handle_command,"Serial"},
        {300,10,20,EVENT_ENABLED,(fnzPointer) handle_command,"Pump"},  // starts 10 seconds after start
        {day/12,handleEvents::toSeconds("0:00:30"),60,EVENT_ENABLED|EVENT_AT_TIME,(fnzPointer) handle_command,"Timed A"},
        {3*day/2,handleEvents::toSeconds("23:59:30"),60,EVENT_ENABLED|EVENT_AT_TIME,(fnzPointer) handle_command,"Timed B"},
        {0,0,30,EVENT_ENABLED|EVENT_MANUAL,(fnzPointer) handle_command,"OnCommand"},	// start on command end after 30 seconds
        {0,0,0,EVENT_MANUAL,(fnzPointer) handle_command,"pointCmd"},  	// start on command
        {0,0,0,0,NULL,NULL}   // terminator
};
handleEvents events(cron);
void handle_atStart(cronStructure &event) {
  events.setTime(day - 50);    // 50" at midnignt
  sprintf(workBuffer,"\n%s SimCron started",events.Time());
  Serial.println(workBuffer);
  delay(100);
  EVENT_DISABLE(event);
  listCommands();
}
void handle_command(cronStructure &event) {
  if (event.name != (char*) "Serial") {
	      sprintf(workBuffer,"%s %s",events.Time(),event.name);
        Serial.print(workBuffer);
        if (event.name == (char*) "Pump" || event.name == (char*)"OnCommand") {
          (IS_EVENT_ENDING(event))? Serial.println(" ended"):Serial.println(" started");
        } else if (event.name == (char*) "pointCmd") { 
            for (int8_t i=0;i<5;i++) {      
              digitalWrite(led, HIGH);  // turn the LED on (HIGH is the voltage level)
              delay(500);               // wait for half second
              digitalWrite(led, LOW);   // turn the LED off by making the voltage LOW
              delay(500);               // wait for a half second
            }
            Serial.println(" executed");
        } else {         
          (IS_EVENT_ENDING(event))? Serial.println(" ended"):Serial.println(" started");
        }
  } else {
     int index = 0;
     while(Serial.available() > 0) {
       if(index < LENGTHWORKBUFFER) {
           workBuffer[index++] = Serial.read();
           workBuffer[index] = '\0';  // Null terminate the string
       }
     } 
     if (index > 0) {
       char inputByte = workBuffer[0]| 0b00100000;
       char printfBuffer[50];
       sprintf(printfBuffer,"%s ",events.Time());
       Serial.print(printfBuffer);
       int8_t iEvent;
       switch (inputByte) {
           case 116:        // t show or set time
              if (index >= 1) {
                int32_t ss = events.toSeconds(workBuffer+1);
                if (ss >= 0) {
                  events.setTime(ss);
                  sprintf(printfBuffer," %lu has changed to %s",ss, events.Time());
                  Serial.println(printfBuffer);
                } else {Serial.print("Incorrect time: ");Serial.println(workBuffer);}
              }
	            break;
            case 99:        // c start/stop on command
            	iEvent = events.searchEvent((char *) "OnCommand");
              if (!IS_EVENT_ENABLED(cron[iEvent])) {
                  EVENT_ENABLE(cron[iEvent]);
                	Serial.println("OnCommand enabled");
              } else {
                cron[iEvent].next = events.clocker(false);		// force close
              	Serial.println("OnCommand disabled");
              }
		          break;
            case 104:        // h list commands
                listCommands();
 		            break;
           case 108:         // l list data
                Serial.println("");
                events.listEvents(cron);
                break;                 
            case 112:        // p exec puntual command
                EVENT_ENABLE(cron[events.searchEvent((char *) "pointCmd")])
                Serial.print("Point command enabled ");
 		            break; 
           default:
               Serial.print(workBuffer);
               Serial.println(" Unknown command");
        }
     }
  }
}
void listCommands() {
    Serial.println("Commands:\n\t h = this help\n\t l = list events\n\t c = start/stop on command\
    \n\t t = set time(thh:mm:ss)\n\t p = start puntual event");
}
void setup() {
  Serial.begin(9600);
  pinMode(led, OUTPUT);
}
void loop() {
  delay(events.croner(cron));    //  handleCronEvents
}
