 /*************************************************************************
 * Cron C++ scheduler
 * Version 1.3
 * 
 * Copyright(c) 2013~2020  El Condor
 * 
 * 15 September 2020
 ************************************************************************/
#include "handleEvents.h"
void handleEvents::setNext(uint32_t &next,uint32_t increment) {   //  set new next if not just assigned
  if (next <= clocker(false)) next = clocker(false) + increment;    // 20/6/2020
}
uint32_t handleEvents::clocker(bool ms) {  // returns time at seconds or at milliseconds if ms=true
  static uint32_t millisOld = 0;
  if (millisOld > millis()) {*ag_Clock += secondsTimerCount;}
  millisOld = millis();
  if (!ms) return (*ag_Clock)+millis()/1000;
  return (*ag_Clock)*1000+millis();
}
uint32_t handleEvents::croner(cronStructure cron[]) {     // condor croner service routine
  uint32_t nextEvent = 0xffffffff;      // maximum
  for (int8_t i=0;cron[i].name != NULL;i++) {
    if (IS_EVENT_ENABLED(cron[i])) {
      if (clocker(false) >= cron[i].next) {
        *ag_millis = millis();
        EVENT_ACTIVE(cron[i]);
        cron[i].fnzAction(&cron[i]);    // call event handler
        if (cron[i].duration > 0) {
          if ((cron[i].status & 6) == 2 ) { // first call of duration event
           setNext(cron[i].next,cron[i].duration);
           cron[i].status = cron[i].status | 4;  // status active duration
         } else {
            setNext(cron[i].next,cron[i].every -  cron[i].duration);    // can be until 49,71026961805556)
            EVENT_INACTIVE(cron[i]);
          }
        } else {    // punctual event
           setNext(cron[i].next,cron[i].every);
           EVENT_INACTIVE(cron[i]);
        }
        if (IS_EVENT_MANUAL(cron[i]) && !IS_EVENT_ACTIVE(cron[i])) {
           EVENT_DISABLE(cron[i]);
        }
      }
      if (IS_EVENT_ENABLED(cron[i])) {if (cron[i].next < nextEvent) nextEvent = cron[i].next;}
    }
  }
  if (nextEvent*1000<clocker(true) || nextEvent == 0xffffffff) *ag_delay = 1000;
  else *ag_delay = nextEvent*1000-clocker(true); // time to delay
  return *ag_delay;
}   
uint8_t handleEvents::searchEvent(char * name) {  // search the event by name, return position
  for (int8_t i=0;ev[i].name != NULL;i++) {if (strcmp(name,ev[i].name) == 0) return i;}
  return -1;
}
void handleEvents::setTime(uint32_t t) {  // sets the time of day
  uint32_t oldTime = clocker(false);
  *ag_Clock = oldTime - (oldTime % day) + t - millis()/1000; // new time
  for (int8_t i=0;ev[i].name != NULL;i++) {     // adjust events time
      if (!IS_EVENT_AT_TIME(ev[i])) ev[i].next = clocker(false) + ev[i].next - oldTime;
      else {
        while (ev[i].next < clocker(false)) ev[i].next += ev[i].every;
      }
  }
}
void handleEvents::addTime(long t) {  // add t seconds to clocker, useful when sleep doesn't update clock
  *ag_Clock = *ag_Clock + t; // new time
}
int32_t handleEvents::toSeconds(const char * time) {    // convert hh[:mm[:ss]] to a number of seconds
  int hh=-1,mm=0,ss=0;
  sscanf(time,"%u:%u:%u",&hh,&mm,&ss);
  if (hh>23||mm>59||ss>59) return -1;
  return (int32_t) hh*3600+mm*60+ss;
}
char* handleEvents::showStatus(cronStructure a) {
  const static char* aStatus[] = {"Disabled","Enabled","Active","Active"};
  static char str[20];
  sprintf(str,"%s%s%s",aStatus[a.status & 3],(IS_EVENT_MANUAL(a)?" Manual":""),IS_EVENT_AT_TIME(a)?" At time":"");
  return str;
}
void handleEvents::listEvents(cronStructure a[]) {
  char wBuffer[80];
  for (int8_t i=0;a[i].name != NULL;i++) {
     sprintf(wBuffer,"%-10s every %6lu for %4lu next %s %s",a[i].name,(unsigned long) a[i].every,(unsigned long) a[i].duration, Time(a[i].next),showStatus(a[i]));
     Serial.println(wBuffer);
     delay(100);
  }
}
char * handleEvents::Time(uint32_t secs) {
  static char timeOfDay[12] = "";
  uint32_t h = (secs % day) / 3600;
  uint32_t m = ((secs % day) - h*3600)/60;
  uint32_t s = secs%60;
  uint32_t d = secs/day;
  sprintf(timeOfDay,"%02lu:%02lu:%02lu+%02lu",(unsigned long) (unsigned long) h,(unsigned long)m,(unsigned long) s,(unsigned long) d);
  return timeOfDay;
}
char * handleEvents::Time() { // real time
  return Time(clocker(false));
}
