/*************************************************************************
 * Cron C++ scheduler
 * Version 1.3
 * 
 * Copyright(c) 2013~2020  El Condor
 * 
 * 16 August 2020
 ************************************************************************/
#ifndef handleEvnts_h
#define handleEvnts_h
#endif
#if defined(ARDUINO) && ARDUINO >= 100
      #include "Arduino.h"
#else
      #include "WProgram.h"
#endif
#define EVENT_MANUAL 8  // it is also disabled
#define EVENT_ENABLED 1
#define EVENT_AT_TIME 16
#define day 86400
#define secondsTimerCount 4294967 // seconds of internal clock
typedef void (* fnzPointer) (struct cronStructure *x);
#define EVENT_ENABLE(a) {a.status = a.status | 1;}
#define EVENT_DISABLE(a) {a.status = a.status & 0b11111110;} // 0111
#define EVENT_ACTIVE(a) {a.status = a.status | 2;}
#define EVENT_INACTIVE(a) {a.status = a.status & 0b11111001;} // 1001
struct cronStructure {
    uint32_t every;    // every seconds
    uint32_t next;     // next event
    uint32_t duration; // for start and stop events (in seconds)
    int8_t status;     // if 1 bit 1: enabled, bit 2: active, bit 3: active ending, bit 4: manual, bit 5: at time
    fnzPointer fnzAction;
    const char* name;
};
inline bool IS_EVENT_ENABLED(cronStructure a) {return a.status & 1;}
inline bool IS_EVENT_ACTIVE(cronStructure a) {return a.status & 2;}
inline bool IS_EVENT_ENDING(cronStructure a) {return a.status & 4;}
inline bool IS_EVENT_MANUAL(cronStructure a) {return a.status & 8;}
inline bool IS_EVENT_AT_TIME(cronStructure a) {return a.status & 16;}
class handleEvents {
  cronStructure *ev;
public:
  handleEvents(cronStructure cron[]) { //the constructor
    int i;  
    for (i=0; cron[i].name != NULL; i++) {}
    if (cron[i].status == 0) {    // border event sanitization
      cron[i].every = 0;
      cron[i].next = 0;           // ag_delay sleep's time
      cron[i].duration = 0;
      ag_Clock = &cron[i].duration;   // 
      ag_delay = &cron[i].next;
      ag_millis = &cron[i].every;
    }
    ev = cron;
    cron[i].status = 1;
  }
  void begin(cronStructure cron[]) {
    begin(cron,0);
  }
  void begin(cronStructure cron[],uint32_t Clock) {
    int i;  
    for (i=0; cron[i].name != NULL; i++) {}   // search event's terminator
    ag_Clock = &cron[i].duration;
    ag_delay = &cron[i].next;
    ag_millis = &cron[i].every;
    Serial.print("\nClock is : ");Serial.println(Clock);
    if (Clock > 0) {
      Serial.print("\nRestart at: ");
      *ag_Clock += (Clock+*ag_millis)/1000;     // for high sleep
      Serial.println(Time());
    }
  }
  uint32_t croner(cronStructure cron[]);
  uint32_t clocker(bool ms);
  void setTime(uint32_t);
  void addTime(long);
  static int32_t toSeconds(const char* time);
  uint8_t searchEvent(char * name);
  void listEvents(cronStructure a[]);
  char * Time();
  char * Time(uint32_t);
  uint32_t *ag_Clock;
  uint32_t *ag_delay;
  uint32_t *ag_millis;
private:
//  volatile uint32_t beforeTime;
  void setNext(uint32_t &next,uint32_t increment);
  char* showStatus(cronStructure a);
};
//
