﻿# =======================================================================================
# Title .........: Serial terminal
# Description ...: Serial terminal for Arduino
# Requirement(s).: PowerShell 3
# Author ........: Giovanni Rossati (El Condor)
# Version........: 0.2.0
# Date...........: 7 March 2015
# =======================================================================================
function onStart() {
	$timer.Interval = 50
	$timer.add_tick({handleSerial})
	$Tooltip = New-Object System.Windows.Forms.ToolTip
	$ToolTip.AutomaticDelay = 100	# milliseconds
	$ToolTip.SetToolTip((getHandle "Com"), "Select a port")
	$ToolTip.SetToolTip((getHandle "command"), "Text to send to Arduino")
}
function handleSerial() {
	$nChar = $port.BytesToRead
	if ($nChar -gt 0) {
		$s = $port.ReadExisting()
		Write-Host -NoNewline $s
		if ((handleOf "save").checked) {$stream.Write($s)}
	}
}
function openCom($DtrEnable = $false) {
	if ([string]::IsNullOrEmpty((valueOf "Com"))) {
		signal("Select a port!")
		return 0
	} else {
		$port.PortName = valueOf "Com"
		$port.BaudRate = 9600
		$port.Parity = "None"
		$port.DataBits =  8
		$port.StopBits = "one"
		$port.ReadTimeout = 100
		$port.DtrEnable = $DtrEnable
		$port.open()
		if ((handleOf "save").checked) {saveOnFile}
		$timer.start()
		Write-Host -ForegroundColor GREEN "PS>" (valueOf "Com") "opened"	
		return 1
	}
}
function signal($info) {
	$wshell = New-Object -ComObject Wscript.Shell
	$res = $wshell.Popup($info,0,"",0x0)
}
function restartCom() {
	if ($port.IsOpen) {$port.close()}
	$timer.stop()
	openCom($true)	# this restarts Arduino (not Leonardo)
	Write-Host -ForegroundColor GREEN "PS>" (valueOf "Com") "opened with DTR ON"
}
function sendCommand() {
	if (!$port.IsOpen) {
		$res = openCom
		if($res -eq 0) {return}
	}
	$command = valueOf "command";
	$port.Write($command + $lineTerminators[(valueOf "lineTerm")]) 
	Write-Host -ForegroundColor GREEN "PS>" $command;
	(handleOf "command").text = ""
}
function saveOnFile() {
	$saveFile = New-Object System.Windows.Forms.SaveFileDialog
#	$saveFile.Filter = "All files (*.*)|*.*"
#	$saveFile.FilterIndex = 2
	$saveFile.FileName = $fileOut
	$saveFile.RestoreDirectory = $true
	$rc = $saveFile.ShowDialog()
	if ($rc -eq [System.Windows.Forms.DialogResult]::OK) {
		$global:stream = [System.IO.StreamWriter] 	$saveFile.FileName	# for capture serial
	} else {(handleOf "save").checked = $false}
}
[void] [System.Reflection.Assembly]::LoadWithPartialName("System.Windows.Forms")
$timer = New-Object System.Windows.Forms.Timer
$fileOut = "captureSerial.txt"
$lineTerminators = @{"CR" = "`r"; "LF" = "`n"; "Both" = "`r`n"; "None" = ""}
$ports = [System.IO.Ports.SerialPort]::getportnames()
$port = new-Object System.IO.Ports.SerialPort
$parm = "list,Com,Communication port,7,,"+(($ports |select -unique) -join "|")
$parm += ";comment,nome,-,,,blue"
$parm += ";R,lineTerm,Line terminator,7,CR,CR|LF|Both|None";
$parm += ";T,command,Command to send,,,button toSend";
$parm += ";CHK,save,Save on file,,On"
$parm += ";B,toSend,Send,,enter,sendCommand"
$parm += ";B,OpenCom,Open,,,openCom"
$parm += ";B,Restart,,,,restartCom"
$parm += ";event,fg_form,Shown:onStart"
.($PSScriptRoot + "\" +"formgen.ps1") 	# load formgen script
formgen $parm -caption "Arduino terminal"
$port.close()
if ($fg_data["save"] -eq "On") {$stream.close()}